#!/usr/bin/perl -w
#
# Copyright (c) 2002 Red Hat, Inc. All rights reserved.
#
# This software may be freely redistributed under the terms of the
# GNU General Public License.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

use Newt;
use Sys::Hostname;
use Digest::MD5;

sub DoubleButton {
    my ($left, $right) = @_;
    
    my $leftb = Newt::Button($left)->Tag($left);
    my $rightb = Newt::Button($right)->Tag($right);

    Newt::Panel(2, 1)
      ->Add(0, 0, $leftb, Newt::NEWT_ANCHOR_RIGHT(), 0, 1, 0, 0)
	  ->Add(1, 0, $rightb, Newt::NEWT_ANCHOR_LEFT(), 1, 1, 0, 0);
}

sub RunForm
{
    my ($panel, $onenter, $onescape) = (@_, "Next", "Cancel");

    $panel->AddHotKey(Newt::NEWT_KEY_ESCAPE());
    $panel->AddHotKey(Newt::NEWT_KEY_ENTER()) unless $onenter eq "Ignore";

    ($reason, $data) = $panel->Run();

    if ($reason eq Newt::NEWT_EXIT_HOTKEY) {
	if ($data == Newt::NEWT_KEY_ESCAPE()) {
	    # They pressed ESCAPE; pretend they pressed "Cancel" or "No"
	    return $onescape;
	}
	elsif ($data == Newt::NEWT_KEY_ENTER()) {
	    my $current = $panel->GetCurrent();
	    if ($panel->{refs}{$$current}->Tag()) {
		# They pressed ENTER over a button; pretend they pressed it.
		return $panel->{refs}{$$current}->Tag();
	    }
	    return $onenter;
	}
    }
    elsif ($reason eq Newt::NEWT_EXIT_COMPONENT) {
	return $data->Tag();
    }
    die "unhandled event ", $reason, " ", $data, "\n";
}

sub start
{
    Newt::Init();
    Newt::Cls();
    Newt::DrawRootText(0, 0,
		       "Red Hat Stronghold Configurator (c) 2002 Red Hat, Inc.");
    Newt::PushHelpLine("  <Tab>/<Alt-Tab> between elements  |" .
		       "  <Space> selects  |" .
		       "  <Escape> to quit");
}

# Run a yes/no dialog
sub yesno
{
    my ($message, $title) = (@_, "Stronghold Configuration");
    my $panel;
    my $ret;

    $panel = Newt::Panel(1, 2, $title);
    
    $panel->Add(0, 0, Newt::TextboxReflowed(50, 10, 10, 0, $message));

    $panel->Add(0, 1, DoubleButton("Yes", "No"));

    $ret = RunForm($panel);

    if ($ret eq "Cancel") {
	Newt::Finished();
	exit 0;
    }
    elsif ($ret eq "Yes") {
	$ret = "YES";
    } 
    else {
	$ret = "NO";
    }

    $panel->Hide();
    undef $panel;

    return $ret;
}

# Generate swish index for Stronghold documentation
sub generate_index
{
    my $panel;
    my $msg = "Building search index for the Stronghold Documentation. " .
	"Please wait, this may take several minutes...";
    
    $panel = Newt::Panel(1, 1, "Building search index");

    $panel->Add(0, 0, Newt::TextboxReflowed(50, 10, 10, 0, $msg));

    $panel->Draw();
    Newt::Refresh();
    
    system("/usr/sbin/generate-index");

    $panel->Hide();
    
    undef $panel;
}

sub replace_phpini
{
    my $ctx = Digest::MD5->new;

    ( -r "/etc/php.ini.rpmnew" ) or return;

    open(FILE, "/etc/php.ini") or return;

    $ctx->addfile(*FILE);

    close FILE;

    # If php.ini is that from vanilla AS installation (just modified
    # by php-ldap), then replace with our php.ini
    if ($ctx->hexdigest eq "4eee648fe71c37779fbd6bac37d3bdee") {
	rename "/etc/php.ini", "/etc/php.ini.sh4save" or return;
	rename "/etc/php.ini.rpmnew", "/etc/php.ini";
    }

    undef $ctx;
}

sub main
{
    my ($tux, $tomcat, $onboot);

    $tux = yesno("Do you wish to enable the Red Hat Content Accelerator (TUX)?");
    $tomcat = yesno("Do you wish to enable the Tomcat Java Servlet Engine?");
    $onboot = yesno("Do you want Stronghold to be run automatically at system startup?");

    # Write out the config file

    if (!open (CONF, ">/etc/sysconfig/httpd")) {
	Newt::newtWinMessage("Error", "Close",
			     "Could not open /etc/sysconfig/httpd");
	Newt::Finished();
	exit 1;
    }
    
    print CONF "# /etc/sysconfig/httpd\n";
    print CONF "# (generated by Stronghold Configurator, /usr/sbin/stronghold-config)\n";
    while ($_ = <DATA>) {
	($tux eq "YES") or s/^TUX/#TUX/g;
	($tomcat eq "YES") or s/^TOMCAT/#TOMCAT/g;
	print CONF $_;
    }
    close(CONF);

    if ($onboot eq "YES") {
	system("/sbin/chkconfig httpd on");
    } else {
	system("/sbin/chkconfig httpd off");
    }

    # Generate swish index.
    generate_index();

    # Replace PHP config file
    replace_phpini();

    if (-f "/var/lock/subsys/httpd") {
	Newt::newtWinMessage("Server running", "Close", 
			     "The HTTP server is already running.");
	
	Newt::Finished();
    }
    else {
	
	Newt::Finished();

	print STDOUT "\n===> Stronghold is now starting up...\n\n";
	
	system("/etc/init.d/httpd start");
    
	print STDOUT "\n===> Stronghold started.\n\n";
	print STDOUT "To connect to your server, start your browser and enter either of these URLs:\n\n";
	print STDOUT "     https://" . hostname() . "/   (secure access)\n";
	print STDOUT "     http://" . hostname() . "/  (standard access)\n\n";
	
    }
}

start();
main();

__DATA__

# TUX is a kernel-based, threaded, extremely high performance HTTP
# server. If enabled here, TUX will start and stop along with Apache,
# which will run with a slightly modified configuration. TUX will then
# serve any serve static files in its document root, passing other
# requests to Apache. See /etc/sysconfig/tux for more TUX
# configuration options.
TUX=YES

# Tomcat is the servlet container that is used in the official
# Reference Implementation for the Java Servlet and JavaServer Pages
# technologies. If enabled here, Tomcat will start and stop along with
# Apache, and mod_webapp (Tomcat's connector module) will be enabled
# in Apache's configuration. See /etc/sysconfig/tomcat for more Tomcat
# configuration options.
TOMCAT=YES
